// Copyright (c) 2015 Siegfried Kettlitz

#include <future.h>

// Future for ADC values.
future_s<uint32_t> ADC_future;

// Future for SerialInput.
future_s<uint8_t> SerialInput_future;

// Interrupt routine.
ISR(TIMER1_OVF_vect)
{
  cli();

  // Read a value and
  volatile static uint32_t sum = 0;
  volatile static uint16_t num = 0;
  sum += digitalRead(A0);
  num++;

  // Check if enough have been collected.
  if( num > 4096 ) {

    // If possible write to future.
    if( ADC_future.accepts_value() )
    {
      ADC_future.set_value( sum );
    }

    // Start over.
    sum = 0;
    num = 0;
  }

  // Check the serial input.
  if( Serial.available() )
  {
    // Set the first.
    if( SerialInput_future.accepts_value() )
    {
      SerialInput_future.set_value( Serial.read() );
    }

    // Flush the rest.
    while( Serial.available() ) {
      Serial.read();
    }
  }

  sei();
}

void setup() {
  // Setup the timer 1 for the interrupt.
  cli();
  TCCR1A = 0;
  TCCR1B = 0;
  TIMSK1 = 0;
  TCCR1A |= (1 << WGM12);
  TCCR1A |= (1 << WGM10);
  TCCR1B |= (1 << CS11);
  TIMSK1 |= (1 << TOIE1);
  sei();

  // Setup serial interface.
  Serial.begin(9600);
  Serial.println( "Start" );
}

void loop() {

  // Check if value is ready.
  if( ADC_future.is_ready() )
  {
    // Get it.
    Serial.print( "ADC has value: " );
    Serial.print( ADC_future.get() );
    Serial.print( " and then " );
    // Wait for the next value.
    ADC_future.wait();
    Serial.print( ADC_future.get() );
    Serial.println( "." );
    Serial.println();

    // Wait for a character to be read.
    Serial.println( "Send a chraracter to continue." );
    SerialInput_future.wait();
    SerialInput_future.get();
    Serial.println( "Done." );
  }

}